package sun.security.x509;

import java.util.Arrays;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.DerInputStream;
import sun.security.util.DerValue;
import sun.security.util.ObjectIdentifier;

public class Extension implements java.security.cert.Extension
{
    protected ObjectIdentifier extensionId;
    protected boolean critical;
    protected byte[] extensionValue;
    private static final int hashMagic = 31;
    
    public Extension() {
        this.extensionId = null;
        this.critical = false;
        this.extensionValue = null;
    }
    
    public Extension(DerValue derValue) throws IOException {
        this.extensionId = null;
        this.critical = false;
        this.extensionValue = null;
        DerInputStream derInputStream = derValue.toDerInputStream();
        this.extensionId = derInputStream.getOID();
        DerValue derValue2 = derInputStream.getDerValue();
        if (derValue2.tag == 1) {
            this.critical = derValue2.getBoolean();
            this.extensionValue = derInputStream.getDerValue().getOctetString();
        }
        else {
            this.critical = false;
            this.extensionValue = derValue2.getOctetString();
        }
    }
    
    public Extension(ObjectIdentifier extensionId, boolean critical, byte[] encoding) throws IOException {
        this.extensionId = null;
        this.critical = false;
        this.extensionValue = null;
        this.extensionId = extensionId;
        this.critical = critical;
        this.extensionValue = new DerValue(encoding).getOctetString();
    }
    
    public Extension(Extension extension) {
        this.extensionId = null;
        this.critical = false;
        this.extensionValue = null;
        this.extensionId = extension.extensionId;
        this.critical = extension.critical;
        this.extensionValue = extension.extensionValue;
    }
    
    public static Extension newExtension(ObjectIdentifier extensionId, boolean critical, byte[] extensionValue) throws IOException {
        Extension extension = new Extension();
        extension.extensionId = extensionId;
        extension.critical = critical;
        extension.extensionValue = extensionValue;
        return extension;
    }
    
    @Override
    public void encode(OutputStream outputStream) throws IOException {
        if (outputStream == null) {
            throw new NullPointerException();
        }
        DerOutputStream derOutputStream = new DerOutputStream();
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream.putOID(this.extensionId);
        if (this.critical) {
            derOutputStream.putBoolean(this.critical);
        }
        derOutputStream.putOctetString(this.extensionValue);
        derOutputStream2.write((byte)48, derOutputStream);
        outputStream.write(derOutputStream2.toByteArray());
        return;
    }
    
    public void encode(DerOutputStream derOutputStream) throws IOException {
        if (this.extensionId == null) {
            throw new IOException("Null OID to encode for the extension!");
        }
        if (this.extensionValue == null) {
            throw new IOException("No value to encode for the extension!");
        }
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putOID(this.extensionId);
        if (this.critical) {
            derOutputStream2.putBoolean(this.critical);
        }
        derOutputStream2.putOctetString(this.extensionValue);
        derOutputStream.write((byte)48, derOutputStream2);
        return;
    }
    
    @Override
    public boolean isCritical() {
        return this.critical;
    }
    
    public ObjectIdentifier getExtensionId() {
        return this.extensionId;
    }
    
    @Override
    public byte[] getValue() {
        return (byte[])this.extensionValue.clone();
    }
    
    public byte[] getExtensionValue() {
        return this.extensionValue;
    }
    
    @Override
    public String getId() {
        return this.extensionId.toString();
    }
    
    @Override
    public String toString() {
        String string = "ObjectId: " + this.extensionId.toString();
        if (this.critical) {
            return string + " Criticality=true\n";
        }
        return string + " Criticality=false\n";
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        if (this.extensionValue != null) {
            byte[] extensionValue = this.extensionValue;
            for (int i = extensionValue.length; i > 0; n += i * extensionValue[--i]) {}
        }
        return (n * 31 + this.extensionId.hashCode()) * 31 + (this.critical ? 1231 : 1237);
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Extension)) {
            return false;
        }
        Extension extension = (Extension)o;
        return this.critical == extension.critical && this.extensionId.equals(extension.extensionId) && Arrays.equals(this.extensionValue, extension.extensionValue);
    }
}
